/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -a dbusmenuadaptor -i /usr/src/ports/libdbusmenu-qt5/libdbusmenu-qt5-0.9.3-0.1.20140604bzr.x86_64/src/libdbusmenu-qt5-0.9.3/src/dbusmenuexporterdbus_p.h -l DBusMenuExporterDBus /usr/src/ports/libdbusmenu-qt5/libdbusmenu-qt5-0.9.3-0.1.20140604bzr.x86_64/src/libdbusmenu-qt5-0.9.3/src/com.canonical.dbusmenu.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 Digia Plc and/or its subsidiary(-ies).
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "dbusmenuadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DbusmenuAdaptor
 */

DbusmenuAdaptor::DbusmenuAdaptor(DBusMenuExporterDBus *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DbusmenuAdaptor::~DbusmenuAdaptor()
{
    // destructor
}

QString DbusmenuAdaptor::status() const
{
    // get the value of property Status
    return qvariant_cast< QString >(parent()->property("Status"));
}

uint DbusmenuAdaptor::version() const
{
    // get the value of property Version
    return qvariant_cast< uint >(parent()->property("Version"));
}

bool DbusmenuAdaptor::AboutToShow(int id)
{
    // handle method call com.canonical.dbusmenu.AboutToShow
    return parent()->AboutToShow(id);
}

void DbusmenuAdaptor::Event(int id, const QString &eventId, const QDBusVariant &data, uint timestamp)
{
    // handle method call com.canonical.dbusmenu.Event
    parent()->Event(id, eventId, data, timestamp);
}

DBusMenuItemList DbusmenuAdaptor::GetGroupProperties(const QList<int> &ids, const QStringList &propertyNames)
{
    // handle method call com.canonical.dbusmenu.GetGroupProperties
    return parent()->GetGroupProperties(ids, propertyNames);
}

uint DbusmenuAdaptor::GetLayout(int parentId, int recursionDepth, const QStringList &propertyNames, DBusMenuLayoutItem &layout)
{
    // handle method call com.canonical.dbusmenu.GetLayout
    return parent()->GetLayout(parentId, recursionDepth, propertyNames, layout);
}

QDBusVariant DbusmenuAdaptor::GetProperty(int id, const QString &name)
{
    // handle method call com.canonical.dbusmenu.GetProperty
    return parent()->GetProperty(id, name);
}


#include "dbusmenuadaptor.moc"
